/*--------------------------------------------------------------------
 * FICHERO:  Proteo.c
 * OBJETIVO: Mostrar en pantalla una secuencia de imgenes contenidas
 *           en ficheros TIFF de 16 colores y hasta 640x480
 * AUTOR:    Pedro Reina
 * FECHA:    L.6.7.1998
 *------------------------------------------------------------------*/

/*--------------------------------------------------------------------
 * Ficheros de cabecera
 *------------------------------------------------------------------*/

#include <Olimpo.h>            /*  Versin 2.0  */
#include <string.h>            /*  strtok()     */

/*--------------------------------------------------------------------
 * Declaracin de funciones
 *------------------------------------------------------------------*/

void PreparaPantalla();

void Explica();
void ExplicaObjetivo();
void ExplicaPrograma();
void ExplicaNombre();
void ExplicaAutor();

void   Imagen();
logico TrataFichero();
logico ExaminaImagen();
void   MuestraImagen();

void   Secuencia();
void   LeeSecuencia();
void   AgregaEnSecuencia();
logico ExaminaSecuencia();
void   MuestraSecuencia();
void   MuestraContinua();
logico MuestraUnaVez();
void   MuestraRapida();

void   Opciones();
octeto PideLapso();

cadena EligeUnNombre();
cadena PideDirectorio();
cadena PideExtension();
cadena PideMascara();
cadena NombreCompleto();

void AsignaPaletaDAC();
void CambiaColorDAC();
void RestauraColor();

logico Tif_LeeLinea();
logico Tif_SituaEnDato();
logico Tif_TablaColor();

/*--------------------------------------------------------------------
 * Definicin de macros constantes
 *------------------------------------------------------------------*/

#define LONG_DIRECTORIO  24
#define LONG_EXTENSION    3
#define LONG_NOMBRE      12
#define LONG_MASCARA     12
#define LONG_DESTINO     20

#define MAX_FICHERO     200

/*--------------------------------------------------------------------
 * Definicin de macros funciones
 *------------------------------------------------------------------*/

#define LimpiaZonaTrabajo()     Pan_Borra(NEGRO,4,0,21,79)
#define LimpiaZonaPequena()     Pan_Borra(NEGRO,10,0,21,79)
#define BorraPantallaVGA()      Pan_Borra(NEGRO,-3,0,27,79)

/*--------------------------------------------------------------------
 * Definicin de variables globales
 *------------------------------------------------------------------*/

octeto Lapso=1;
lista  Diapo;    /* Lista con los nombres de los ficheros TIFF */
lista  Tiempo;   /* Lista con los tiempos de cada diapositiva  */
caracter DirectorioSecuencia[50];

/*--------------------------------------------------------------------
 * Programa principal
 *------------------------------------------------------------------*/
void main (Narg, Arg)
int   Narg;
char *Arg[];
  {
  enum { SALIDA_ESC, SALIDA, EXPLICA, IMAGEN, SECUENCIA, OPCIONES };

  static cadena Principal[] = {"Sa>lida", ">Explica", ">Imagen", ">Secuencia",
                               ">Opciones", NIL };
  entero Opcion;
  logico Sigue=SI;
  cadena Nombre;
  octeto k;

  Pan_Define (PAN_GRAFICO);

  PreparaPantalla();

  if ( Pan_Modo() == PAN_TEXTO )
    { Usr_ErrorFatal ("Error fatal: este ordenador no dispone de VGA"); }

  for ( k=0 ; k<16 ; k++ )
    { AsignaPaletaDAC (k,k); }

  if ( Narg > 1 )
    {
    Nombre = Cad_Duplica ( Arg[1] );
    if ( Fch_Existe (Nombre) )
      {
      LeeSecuencia (Nombre);
      if ( ExaminaSecuencia() )
        {
        MuestraContinua();
        RestauraColor();
        BorraPantallaVGA();
        PreparaPantalla();
        }
      }
    Cad_Destruye (Nombre);
    }

  Opcion = SECUENCIA;
  while ( Sigue )
    {
    Opcion = Men_Horizontal (2,2,77,Principal,Opcion);
    switch ( Opcion )
      {
      case SALIDA_ESC:
      case SALIDA:
           if ( Usr_Consulta ("Quieres terminar el programa?") )
             { Sigue = NO; }
           Opcion = SALIDA;
           break;
      case EXPLICA:    Explica();          break;
      case IMAGEN:     Imagen();           break;
      case SECUENCIA:  Secuencia();        break;
      case OPCIONES:   Opciones();         break;
      }
    LimpiaZonaTrabajo();
    }

  Pan_Cierra();
  }

/*--------------------------------------------------------------------
 * Definicin de funciones
 *------------------------------------------------------------------*/

/*--------------------------------------------------------------------
 * FUNCION:  PreparaPantalla
 * OBJETIVO: Escribir en la pantalla los datos esenciales
 *------------------------------------------------------------------*/
void PreparaPantalla()
  {
  Prg_Presenta ("Proteo", "1.1", "Pedro Reina", "Julio 1998");
  Cdr_Caja (CDR_SIMPLE,1,0,3,79,NEGRO,BLANCO);
  }

/*--------------------------------------------------------------------
 * Funciones del men Explica
 *------------------------------------------------------------------*/

/*--------------------------------------------------------------------
 * FUNCION:  Explica()
 * OBJETIVO: Dirigir las explicaciones
 *------------------------------------------------------------------*/
void Explica()
  {
  enum { SALIDA_ESC, OBJETIVO, PROGRAMA, NOMBRE, AUTOR };
  static contador MaxOpcion = 4;

  static cadena Menu[] = {">Objetivo", ">Programa", ">Nombre",
                          ">Autor", NIL };

  static cadena Presentacion[] = {
    "Puedes pedir explicacin sobre diferentes aspectos",
    "de este programa. Elige el que desees y pulsa ESC",
    "cuando hayas terminado.",
    NIL };

  logico   Sigue = SI;
  entero   Opcion = 1;
  contador i;

  while ( Sigue )
    {
    LimpiaZonaTrabajo();
    Pan_Color (NEGRO,BLANCO);
    for ( i=0 ; Presentacion[i] ; i++ )
      { Pan_PonTexto (6+i,15,Presentacion[i]); }

    Opcion = Men_Vertical (11,34,14,50,Menu,Opcion);
    LimpiaZonaTrabajo();
    switch ( Opcion )
      {
      case SALIDA_ESC:  Sigue = NO;         break;
      case OBJETIVO:    ExplicaObjetivo();  break;
      case PROGRAMA:    ExplicaPrograma();  break;
      case NOMBRE:      ExplicaNombre();    break;
      case AUTOR:       ExplicaAutor();     break;
      }
    Opcion++;
    if ( Opcion > MaxOpcion )  { Opcion = 1; }
    }
  }

/*--------------------------------------------------------------------
 * FUNCION:  ExplicaObjetivo()
 * OBJETIVO: Explicar el objetivo principal del programa
 *------------------------------------------------------------------*/
void ExplicaObjetivo()
  {
  static cadena Explicacion[] = {
    "Este programa permite ver una secuencia de imgenes",
    "almacenadas cada una en un fichero TIFF. Las imgenes",
    "deben ser de 16 colores y con unas dimensiones",
    "mximas de 640x480.",
    "",
    "Las secuencias se almacenan en ficheros de texto, lo que",
    "permite su modificacin con cualquier editor.",
    "",
    "Para ver explicaciones ms detalladas que las presentadas",
    "en pantalla, consulta el fichero de documentacin, que es",
    "un simple fichero de texto.",
    NIL };

  contador i;

  Pan_Color (NEGRO,BLANCO);
  for ( i=0 ; Explicacion[i] ; i++ )
    { Pan_PonTexto (5+i,10,Explicacion[i]); }

  Usr_PulsaUnaTecla ("");
  }

/*--------------------------------------------------------------------
 * FUNCION:  ExplicaPrograma()
 * OBJETIVO: Explicar el funcionamiento del programa
 *------------------------------------------------------------------*/
void ExplicaPrograma()
  {
  static cadena Explicacion[] = {
    "El programa est dirigido por mens. El principal es el men",
    "horizontal que siempre est presente en la parte superior de",
    "la pantalla.",
    "",
    "Con la opcin 'Imagen' puedes ver en pantalla los datos internos",
    "de una imagen y visualizarla.",
    "",
    "Con la opcin 'Secuencia' puedes cargar una secuencia y visualizarla",
    "de tres formas distintas: 'Continua' (se repite indefinidamente),",
    "'Una vez' (se muestra la secuencia tal cual) y 'Rpida' (cada imagen",
    "aparece el nmero de segundos especificado en Lapso).",
    "",
    "En Opciones se pueden definir dos cosas: si el sonido est",
    "encendido o apagado y el Lapso usado en visualizacin rpida",
    "de secuencias.",
    NIL };

  contador i;

  Pan_Color (NEGRO,BLANCO);
  for ( i=0 ; Explicacion[i] ; i++ )
    { Pan_PonTexto (5+i,6,Explicacion[i]); }

  Usr_PulsaUnaTecla ("");
  }

/*--------------------------------------------------------------------
 * FUNCION:  ExplicaAutor()
 * OBJETIVO: Explicar quin es el autor y cmo contactar con l
 *------------------------------------------------------------------*/
void ExplicaAutor()
  {
  static cadena Mensaje[] = {
    "Este programa es de Dominio Pblico. Lo puedes usar, distribuir",
    "y modificar como desees.",
    "",
    "Ha sido escrito en C usando el sistema de programacin Olimpo;",
    "si no dispones de la versin 2.0 de Olimpo, puedes cargarla",
    "directamente desde mi sede web o pedrmela si no dispones de",
    "acceso a Internet.",
    "",
    "Si tienes cualquier duda o consulta, cuntamela e intentar ayudarte.",
    "",
    "Mis datos:",
    "",
    "Pedro Reina",
    "c/ Marquesa de Argeso, 4      Telfono: 91 565 17 59",
    "28019 Madrid                   Correo electrnico: pedro@anit.es",
    "Espaa                         Web: www.anit.es/pedro",
    NIL };

  octeto i;

  Pan_Color (NEGRO,BLANCO);
  for ( i=0 ; Mensaje[i] ; i++ )
    { Pan_PonTexto (5+i,5,Mensaje[i]); }

  Usr_PulsaUnaTecla ("");
  }

/*--------------------------------------------------------------------
 * FUNCION:  ExplicaNombre()
 * OBJETIVO: Explicar el nombre del programa
 *------------------------------------------------------------------*/
void ExplicaNombre()
  {
  static cadena Explicacion[] = {
    "Proteo es un dios del mar que generalmente vive en la",
    "isla de Faros, no lejos de la desembocadura del Nilo.",
    "Est dotado de la virtud de metamorfosearse en",
    "cualquier forma que desee: puede convertirse no slo en",
    "animal, sino en elemento, tal como el agua y el fuego.",
    "En una ocasin, delante de Menelao, se metamorfose",
    "sucesivamente en len, en serpiente, en pantera, en un",
    "enorme jabal, en agua y en rbol.",
    "",
    "Diccionario de mitologa griega y romana",
    "Pierre Grimal, Editorial Paidos",
    NIL };
  contador i;

  Pan_Color (NEGRO,BLANCO);
  for ( i=0 ; Explicacion[i] ; i++ )
    { Pan_PonTexto (5+i,10,Explicacion[i]); }

  Usr_PulsaUnaTecla ("");
  }

/*--------------------------------------------------------------------
 * Funciones del men Imagen
 *------------------------------------------------------------------*/

/*--------------------------------------------------------------------
 * FUNCION:  Imagen()
 * OBJETIVO: Dirigir la visualizacin de ficheros TIFF
 * ENTRADAS: Ninguna
 * SALIDAS:  Ninguna
 *------------------------------------------------------------------*/
void Imagen()
  {
  enum { SALIDA_ESC, MUESTRA, DIRECTORIO, MASCARA, EXTENSN };

  static cadena MenuFichero[] = {">Mostrar", ">Directorio",
                                 ">Mscara", ">Extensin", NIL };

  static char Directorio [LONG_DIRECTORIO+1];
  static char Nombre [LONG_NOMBRE+1];
  static char Mascara [LONG_MASCARA+1] = "*";
  static char Extension [LONG_EXTENSION+1] = "tif";

  cadena   Aux;
  entero   Opcion;
  contador Fil=4, Col=20, Ancho=40, Alto=5;
  logico   Sigue = SI, Refresca = SI;

  Opcion = MUESTRA;
  while ( Sigue )
    {
    if ( Refresca )
      {
      Cdr_Caja (CDR_SIMPLE,Fil,Col,Fil+Alto,Col+Ancho,NEGRO,VERDE);
      Pan_Borra (NEGRO,Fil+2,Col+15,Fil+Alto-1,Col+Ancho-1);
      Pan_Color (NEGRO,VERDE);
      Pan_PonTexto (Fil+2,Col+15,Directorio);
      Pan_PonTexto (Fil+3,Col+15,Mascara);
      Pan_PonTexto (Fil+4,Col+15,Extension);
      Refresca = NO;
      }

    Opcion = Men_Vertical (Fil+1,Col+1,Fil+Alto-1,Col+13,MenuFichero,Opcion);
    switch ( Opcion )
      {
      case SALIDA_ESC:  Sigue = NO;      break;

      case MUESTRA:     Aux = EligeUnNombre (Directorio, Mascara,
                                             Extension);
                        if ( Aux )
                          {
                          if ( Cad_Longitud(Aux) > LONG_NOMBRE )
                            { Aux[LONG_NOMBRE] = NULO; }
                          Cad_Copia (Nombre, Aux);
                          Cad_Destruye (Aux);
                          if ( TrataFichero (Directorio,Nombre,Extension) )
                            { Sigue = NO; }
                          else
                            {
                            Refresca = SI;
                            LimpiaZonaTrabajo();
                            }
                          }
                        break;

      case DIRECTORIO:  Aux = PideDirectorio(Directorio);
                        if ( Tec_Ultima() != TEC_ESC )
                          {
                          Cad_Copia (Directorio,Aux);
                          Refresca = SI;
                          }
                        Cad_Destruye (Aux);
                        break;

      case EXTENSN:     Aux = PideExtension(Extension);
                        if ( Tec_Ultima() != TEC_ESC )
                          {
                          Cad_Copia (Extension,Aux);
                          Refresca = SI;
                          }
                        Cad_Destruye (Aux);
                        break;

      case MASCARA:     Aux = PideMascara(Mascara);
                        if ( Tec_Ultima() != TEC_ESC )
                          {
                          Cad_Copia (Mascara,Aux);
                          Refresca = SI;
                          }
                        Cad_Destruye (Aux);
                        break;
      }
    LimpiaZonaPequena();
    }
  }

/*--------------------------------------------------------------------
 * FUNCION:  TrataFichero()
 * OBJETIVO: Preparar la visualizacin de un fichero TIFF
 * ENTRADAS: Directorio, nombre y extensin del fichero
 * SALIDAS:  Lgica, que indica que el usuario ha visto la imagen
 *------------------------------------------------------------------*/
logico TrataFichero (Dir,Nom,Ext)
cadena Dir,Nom,Ext;
  {
  cadena Nombre;
  logico Respuesta = NO;

  Nombre = NombreCompleto(Dir,Nom,Ext);

  if ( ExaminaImagen (Nombre) &&
       Usr_Consulta ("Quieres ver la imagen?") )
    {
    MuestraImagen (Nombre,0);
    RestauraColor();
    BorraPantallaVGA();
    PreparaPantalla();
    Respuesta = SI;
    }

  Cad_Destruye (Nombre);

  return ( Respuesta );
  }

/*--------------------------------------------------------------------
 * FUNCION:  ExaminaImagen()
 * OBJETIVO: Examinar un fichero TIFF y ver si el programa lo puede tratar
 * ENTRADAS: Nombre completo del fichero
 * SALIDAS:  Lgica, que indica si la imagen se puede tratar
 *------------------------------------------------------------------*/
logico ExaminaImagen (Nombre)
cadena Nombre;
  {
  tiff     Tiff;
  logico   Vale;
  entero   Ancho, Alto;
  octeto   i, j, Fila;
  contador Compresion, BitsPorPixel;
  contpos  Color[16][3];

  LimpiaZonaTrabajo();
  Pan_Color (NEGRO, VERDE);
  Pan_PonTexto (5,3,"Nombre del fichero: ");
  Pan_Tinta (BLANCO);
  Pan_Texto (Nombre);

  if ( Tiff = Tif_Abre (Nombre) )
    {
    Ancho = Tif_Ancho (Tiff);
    Alto  = Tif_Alto (Tiff);
    Pan_Tinta (VERDE);
    Pan_PonTexto (7,3,"Dimensiones: ");
    Pan_Tinta (BLANCO);
    Pan_Entero (Ancho,1);
    Pan_Texto (" x ");
    Pan_Entero (Alto,1);

    Compresion = Tif_Compresion ( Tiff );
    Pan_Tinta (VERDE);
    Pan_PonTexto (9,3,"Modo de compresion: ");
    Pan_Tinta (BLANCO);
    Pan_Entero (Compresion,1);

    BitsPorPixel = Tif_BitsPorPixel ( Tiff );
    Pan_Tinta (VERDE);
    Pan_PonTexto (11,3,"Bits por pixel: ");
    Pan_Tinta (BLANCO);
    Pan_Entero (BitsPorPixel,1);

    if ( Tif_TablaColor ( Tiff, Color ) )
      {
      Pan_Tinta (VERDE);
      Pan_PonTexto (5,40,"N color   Rojo     Verde    Azul");
      Pan_Tinta (BLANCO);
      for ( i=0 ; i<16 ; i++ )
        {
        Pan_PonEntero (6+i,44,i,1);
        for ( j=0 ; j<3 ; j++ )
          {
          Color[i][j] >>= 10;
          Pan_PonEntero (6+i,52+9*j,Color[i][j],1);
          }
        }
      }

    Pan_Color (NEGRO,BLANCO);
    Vale = SI;
    Fila = 13;

    if ( Ancho > 640 )
      {
      Pan_PonTexto (Fila++, 3, "No se admite la anchura");
      Vale = NO;
      }

    if ( Alto > 480 )
      {
      Pan_PonTexto (Fila++, 3, "No se admite la altura");
      Vale = NO;
      }

    if ( BitsPorPixel != 4 )
      {
      Pan_PonTexto (Fila++, 3, "No se admite el nmero de colores");
      Vale = NO;
      }

    if ( Compresion != 1 )
      {
      Pan_PonTexto (Fila++, 3, "No se admite el modo de compresion");
      Vale = NO;
      }

    Tif_Cierra (Tiff);
    }

  if ( !Vale )  { Usr_Avisa ("No puedo manejar esta imagen"); }

  return (Vale);
  }

/*--------------------------------------------------------------------
 * FUNCION:  MuestraImagen()
 * OBJETIVO: Mostrar un fichero TIFF
 * ENTRADAS: Nombre completo del fichero y el tiempo que debe permanecer
 *           en la pantalla (0 significa hasta que se pulse una tecla)
 * SALIDAS:  Ninguna
 *------------------------------------------------------------------*/
void MuestraImagen (Nombre, Crono)
cadena Nombre;
octeto Crono;
  {
  contpos  Color[16][3];
  contador i, j, Ancho, Alto;
  octeto   Linea[640];
  tiff     Tiff;

  if ( Tiff = Tif_Abre (Nombre) )
    {
    Ancho = Tif_Ancho (Tiff);
    Alto = Tif_Alto (Tiff);

    Tif_TablaColor (Tiff, Color);
    for ( i=0 ; i<16 ; i++ )
      {
      for ( j=0 ; j<3 ; j++ )
        { Color[i][j] >>= 10; }
      CambiaColorDAC ((octeto)i, (octeto) Color[i][0],
                     (octeto) Color[i][1], (octeto) Color[i][2] );
      }

    Tif_SituaEnDato (Tiff);
    for ( i=0 ; i<Alto ; i++ )
      {
      Tif_LeeLinea (Tiff, Linea);
      for ( j=0 ; j<Ancho ; j++ )
        { Zon_PixelFisico (j, i, (contador) Linea[j]); }
      for ( j=Ancho ; j<640 ; j++ )
        { Zon_PixelFisico (j, i, 0 ); }
      }
    for ( i=Alto ; i<480 ; i++ ) {
    for ( j=0 ; j<80 ; j++ )     {
      Zon_OxelFisico (j, i, 0xFF, 0 );
      }}

    Tif_Cierra ( Tiff );

    if ( Crono )  { Tim_Espera ((real)Crono); }
    else          { Tec_Pulsada(); }
    }
  }

/*--------------------------------------------------------------------
 * Funciones del men Secuencia
 *------------------------------------------------------------------*/

/*--------------------------------------------------------------------
 * FUNCION:  Secuencia()
 * OBJETIVO: Dirigir la gestin de secuencias
 * ENTRADAS: Ninguna
 * SALIDAS:  Ninguna
 *------------------------------------------------------------------*/
void Secuencia()
  {
  enum { SALIDA_ESC, MUESTRA, DIRECTORIO, MASCARA, EXTENSN };

  static cadena MenuEstilo[] = {">Muestra", ">Directorio",
                                ">Mscara", ">Extensin", NIL };

  static char Directorio [LONG_DIRECTORIO+1];
  static char Nombre [LONG_NOMBRE+1];
  static char Mascara [LONG_MASCARA+1] = "*";
  static char Extension [LONG_EXTENSION+1] = "sec";

  cadena   Aux;
  entero   Opcion;
  contador Fil=4, Col=20, Ancho=40, Alto=5;
  logico   Sigue = SI, Refresca = SI;

  Opcion = MUESTRA;
  while ( Sigue )
    {
    if ( Refresca )
      {
      Cdr_Caja (CDR_SIMPLE,Fil,Col,Fil+Alto,Col+Ancho,NEGRO,VERDE);
      Pan_Borra (NEGRO,Fil+2,Col+15,Fil+Alto-1,Col+Ancho-1);
      Pan_Color (NEGRO,VERDE);
      Pan_PonTexto (Fil+2,Col+15,Directorio);
      Pan_PonTexto (Fil+3,Col+15,Mascara);
      Pan_PonTexto (Fil+4,Col+15,Extension);
      Refresca = NO;
      }

    Opcion = Men_Vertical (Fil+1,Col+1,Fil+Alto-1,Col+13,MenuEstilo,Opcion);
    switch ( Opcion )
      {
      case SALIDA_ESC:  Sigue = NO;      break;

      case MUESTRA:     Aux = EligeUnNombre (Directorio, Mascara,
                                             Extension);
                        if ( Aux )
                          {
                          if ( Cad_Longitud(Aux) > LONG_NOMBRE )
                            { Aux[LONG_NOMBRE] = NULO; }
                          Cad_Copia (Nombre, Aux);
                          Cad_Destruye (Aux);

                          Aux = NombreCompleto (Directorio,Nombre,Extension);
                          LeeSecuencia (Aux);
                          if ( ExaminaSecuencia() )
                            { MuestraSecuencia (Aux); }
                          Cad_Destruye (Aux);
                          }
                        Sigue = NO;
                        break;

      case DIRECTORIO:  Aux = PideDirectorio(Directorio);
                        if ( Tec_Ultima() != TEC_ESC )
                          {
                          Cad_Copia (Directorio,Aux);
                          Refresca = SI;
                          }
                        Cad_Destruye (Aux);
                        break;

      case EXTENSN:     Aux = PideExtension(Extension);
                        if ( Tec_Ultima() != TEC_ESC )
                          {
                          Cad_Copia (Extension,Aux);
                          Refresca = SI;
                          }
                        Cad_Destruye (Aux);
                        break;

      case MASCARA:     Aux = PideMascara(Mascara);
                        if ( Tec_Ultima() != TEC_ESC )
                          {
                          Cad_Copia (Mascara,Aux);
                          Refresca = SI;
                          }
                        Cad_Destruye (Aux);
                        break;
      }
    LimpiaZonaPequena();
    }
  }

/*--------------------------------------------------------------------
 * FUNCION:  LeeSecuencia()
 * OBJETIVO: Leer un fichero con la definicin de una secuencia
 * ENTRADAS: El nombre del fichero
 * SALIDAS:  Ninguna
 *------------------------------------------------------------------*/
void LeeSecuencia (Nombre)
cadena Nombre;
  {
  config Config;
  cadena Linea;
  char  *Etiqueta, *Aux, *Aux2;

  Usr_Informa ("Leyendo fichero de secuencia");

  Cad_Copia (DirectorioSecuencia, "");
  if ( Diapo )
    {
    Lis_Destruye (Diapo);
    Diapo = NIL;
    Lis_Destruye (Tiempo);
    Tiempo = NIL;
    }

  if ( Config = Cnf_Abre (Nombre) )
    {
    while ( Linea = Cnf_Lee (Config) )
      {
      if ( Etiqueta = strtok (Linea, " ") )
        {

        if ( Cad_Igual (Etiqueta,"Directorio") )
          {
          if ( Aux = strtok (NULL, " ") )
            { Cad_Copia (DirectorioSecuencia, Aux); }
          }

        else
          {
          if ( Aux = strtok (NULL, "\0") )
            { AgregaEnSecuencia (Etiqueta, Aux); }
          }

        } /* Fin if Etiqueta */
      Cad_Destruye (Linea);
      } /* Fin while Linea */
    } /* Fin if Cnf_Abre() */
  }

/*--------------------------------------------------------------------
 * FUNCION:  AgregaEnSecuencia()
 * OBJETIVO: Aadir a la secuencia otro elemento
 * ENTRADAS: Dos cadenas con la duracion y el nombre
 * SALIDAS:  Ninguna
 *------------------------------------------------------------------*/
void AgregaEnSecuencia (Duracion, Nombre)
cadena Duracion, Nombre;
  {
  octeto *Num;

  Num = malloc (sizeof(octeto));
  *Num = atoi (Duracion);

  Nombre = Nombre + Cad_PrimerUtil(Nombre) - 1;

  if ( Diapo )
    {
    Lis_AgregaFin (Diapo, Cad_Duplica(Nombre));
    Lis_AgregaFin (Tiempo, Num);
    }
  else
    {
    Diapo = Lis_Crea();
    Lis_PonContenido (Diapo, Cad_Duplica(Nombre));
    Tiempo = Lis_Crea();
    Lis_PonContenido (Tiempo, Num);
    }
  }

/*--------------------------------------------------------------------
 * FUNCION:  ExaminaSecuencia()
 * OBJETIVO: Examinar cada fichero de una secuencia
 * ENTRADAS: Ninguna, se usan las variables globales Diapo
 *           y DirectorioSecuencia
 * SALIDAS:  Lgica, que indica si la secuencia se puede tratar
 *------------------------------------------------------------------*/
logico ExaminaSecuencia()
  {
  lista  L;
  logico Respuesta=SI;
  cadena Nombre;

  Usr_Informa ("Estudiando secuencia");
  for ( L=Diapo ; L && Respuesta ; L=Lis_Siguiente(L) )
    {
    Nombre = NombreCompleto (DirectorioSecuencia, Lis_Contenido(L), "tif");
    Respuesta = ExaminaImagen (Nombre);
    Cad_Destruye (Nombre);
    }

  return ( Respuesta );
  }

/*--------------------------------------------------------------------
 * FUNCION:  MuestraSecuencia()
 * OBJETIVO: Dirigir el modo de mostrar una secuencia
 * ENTRADAS: El nombre del fichero que contiene la secuencia
 * SALIDAS:  Ninguna
 *------------------------------------------------------------------*/
void MuestraSecuencia (Nombre)
cadena Nombre;
  {
  enum { SALIDA_ESC, CONTINUA, UNAVEZ, RAPIDA };
  static cadena Menu[] = {">Continua", ">Una vez", ">Rpida", NIL };
  entero Opcion = CONTINUA;

  LimpiaZonaTrabajo();
  Pan_Color (NEGRO,VERDE);
  Pan_PonTexto (6,15,"Nombre de la secuencia: ");
  Pan_Tinta (BLANCO);
  Pan_Texto (Nombre);
  Pan_PonTexto (8,15,"Elige el modo de presentacin de la secuencia");
  Pan_PonTexto (17,15,"Para parar la secuencia podrs pulsar ESC");

  Cdr_Caja (CDR_SIMPLE, 10,29,14,47,NEGRO,VERDE);
  Opcion = Men_Vertical (11,33,13,46,Menu,Opcion);
  switch ( Opcion )
    {
    case CONTINUA:    MuestraContinua();  break;
    case UNAVEZ:      MuestraUnaVez();    break;
    case RAPIDA:      MuestraRapida();    break;
    }

  if ( Opcion )
    {
    RestauraColor();
    BorraPantallaVGA();
    PreparaPantalla();
    }
  }

/*--------------------------------------------------------------------
 * FUNCION:  MuestraContinua()
 * OBJETIVO: Mostrar una secuencia cclicamente
 * ENTRADAS: Ninguna
 * SALIDAS:  Ninguna
 *------------------------------------------------------------------*/
void MuestraContinua()
  {
  while ( MuestraUnaVez() )
    ;
  }

/*--------------------------------------------------------------------
 * FUNCION:  MuestraUnaVez()
 * OBJETIVO: Mostrar una secuencia una sola vez
 * ENTRADAS: Ninguna, se usan las variables globales Diapo, Tiempo
 *           y DirectorioSecuencia
 * SALIDAS:  Lgica, que indica si ha terminado la secuencia normalmente.
 *           Si el usuario la interrumpe pulsando ESC, se devuelve NO
 *------------------------------------------------------------------*/
logico MuestraUnaVez()
  {
  lista  L, T;
  logico Sigue=SI;
  cadena Nombre;

  for ( L=Diapo , T=Tiempo ;
        L && Sigue;
        L=Lis_Siguiente(L) , T=Lis_Siguiente(T) )
    {
    Nombre = NombreCompleto (DirectorioSecuencia, Lis_Contenido(L), "tif");
    MuestraImagen (Nombre, *(octeto *)Lis_Contenido(T));
    Cad_Destruye (Nombre);
    if ( Tec_Disponible() )
      { Sigue = Tec_Pulsada() != TEC_ESC; }
    }

  return ( Sigue );
  }

/*--------------------------------------------------------------------
 * FUNCION:  MuestraRapida()
 * OBJETIVO: Mostrar una secuencia rpidamente
 * ENTRADAS: Ninguna, se usan las variables globales Diapo, Lapso
 *           y DirectorioSecuencia
 * SALIDAS:  Ninguna
 *------------------------------------------------------------------*/
void MuestraRapida()
  {
  lista  L;
  logico Sigue=SI;
  cadena Nombre;

  for ( L=Diapo ; L && Sigue; L=Lis_Siguiente(L) )
    {
    Nombre = NombreCompleto (DirectorioSecuencia, Lis_Contenido(L), "tif");
    MuestraImagen (Nombre, Lapso);
    Cad_Destruye (Nombre);
    if ( Tec_Disponible() )
      { Sigue = Tec_Pulsada() != TEC_ESC; }
    }
  }

/*--------------------------------------------------------------------
 * Funciones del men Opciones
 *------------------------------------------------------------------*/

/*--------------------------------------------------------------------
 * FUNCION:  Opciones()
 * OBJETIVO: Seleccionar las opciones del programa
 * ENTRADAS: Ninguna
 * SALIDAS:  Ninguna
 *------------------------------------------------------------------*/
void Opciones()
  {
  enum { SALIDA_ESC, SONIDO, LAPSO};

  static cadena MenuOpc[] = {">Sonido", ">Lapso", NIL };

  entero   Opcion;
  contador Fil=4, Col=56, Ancho=23, Alto=3;
  logico   Sigue = SI;

  Cdr_Caja (CDR_SIMPLE,Fil,Col,Fil+Alto,Col+Ancho,NEGRO,VERDE);

  Opcion = SONIDO;
  while ( Sigue )
    {
    Pan_Color (NEGRO,VERDE);

    if ( Son_Estado() ) { Pan_PonTexto (Fil+1,Col+12,"Encendido"); }
    else                { Pan_PonTexto (Fil+1,Col+12,"Apagado  "); }
    Pan_PonEntero (Fil+2,Col+12,Lapso,1);

    Opcion = Men_Vertical (Fil+1,Col+1,Fil+Alto-1,Col+11,MenuOpc,Opcion);
    switch ( Opcion )
      {
      case SALIDA_ESC:    Sigue = NO;      break;
      case SONIDO:        Son_Cambia();    break;
      case LAPSO:         Lapso = PideLapso (Lapso);
                          break;
      }
    }
  }

/*--------------------------------------------------------------------
 * FUNCION:  PideLapso()
 * OBJETIVO: Pedir al usuario el lapso entre dos imgenes en secuencia
 *           rpida
 * ENTRADAS: El lapso propuesto
 * SALIDAS:  El nuevo lapso
 *------------------------------------------------------------------*/
octeto PideLapso (Lapso)
octeto Lapso;
  {
  octeto Respuesta;
  static contador Fil=12, Col=30, Ancho=20, Alto=4;

  Cdr_Caja (CDR_DOBLE,Fil,Col,Fil+Alto,Col+Ancho,NEGRO,VERDE);

  Pan_Color (NEGRO,VERDE);
  Pan_PonTexto (Fil+1,Col+3,"Elige el lapso:");
  Respuesta = Usr_Entero (Lapso, 1, 1, 9, Fil+3, Col+10, NEGRO, BLANCO);
  Pan_Borra (NEGRO,Fil,Col,Fil+Alto,Col+Ancho);

  return ( Respuesta );
  }

/*--------------------------------------------------------------------
 * Funciones de apoyo para elegir un fichero
 *------------------------------------------------------------------*/

/*--------------------------------------------------------------------
 * FUNCION:  EligeUnNombre()
 * OBJETIVO: Mostrar al usuario una lista con los ficheros que puede leer
 *           y devolver el que elija
 * ENTRADAS: Directorio, mscara para el nombre y extensin del fichero
 * SALIDAS:  Una cadena con el nombre (sin directorio ni extensin) o NIL
 * NOTA:     La cadena devuelta hay que destruirla cuando no sea necesaria
 *------------------------------------------------------------------*/
cadena EligeUnNombre (Dir,Nom,Ext)
cadena Dir,Nom,Ext;
  {
  contador Fil=10, Col=25, Ancho=26, Alto=11;
  cadena   Mascara, Mensaje, Respuesta = NIL;
  cadena   MenuPosible [MAX_FICHERO+1];
  lista    Posible, L;
  contador i, Elegido;
  logico   Sigue;

  LimpiaZonaPequena();
  Cdr_Caja (CDR_SIMPLE,Fil,Col,Fil+Alto,Col+Ancho,NEGRO,VERDE);

  Mascara = NombreCompleto(Dir,Nom,Ext);
  Usr_Informa ("Buscando ficheros");
  Posible = Fch_ListaNombre (Mascara);
  if ( Posible )
    {
    for ( i=0 , L=Posible ; L && i<MAX_FICHERO ; L=Lis_Siguiente(L) )
      { MenuPosible[i++] = Lis_Contenido(L); }
    MenuPosible[i] = NIL;

    Pan_Color (NEGRO,VERDE);
    Pan_PonTexto (Fil+1,Col+4,"Elije el fichero:");
    if ( Elegido =
         Men_Vertical (Fil+3,Col+4,Fil+Alto-1,Col+20,MenuPosible,(entero)1) )
      {
      Sigue = SI;
      for ( i=Cad_Longitud(MenuPosible[Elegido-1]) ; i>=0 && Sigue ; i-- )
        {
        if ( MenuPosible[Elegido-1][i] == Fch_Separador() )
          {
          MenuPosible[Elegido-1][i] = NULO;
          Sigue = NO;
          }
        }
      Respuesta = Cad_Duplica (MenuPosible[Elegido-1]);
      }
    Lis_Destruye (Posible);
    }

  else
    {
    Mensaje = Cad_Une ("No se encontrado ningn fichero ","\"",Mascara,
                       "\"", NIL);
    Usr_PulsaUnaTecla (Mensaje);
    Cad_Destruye (Mensaje);
    }

  Cad_Destruye (Mascara);
  return ( Respuesta );
  }

/*--------------------------------------------------------------------
 * FUNCION:  NombreCompleto()
 * OBJETIVO: Devolver el nombre completo de un fichero
 * ENTRADAS: Directorio, nombre y extensin del fichero
 * SALIDAS:  Una cadena con el nombre completo
 * NOTA:     La cadena hay que destruirla cuando no sea necesaria
 *------------------------------------------------------------------*/
cadena NombreCompleto (Dir,Nom,Ext)
cadena Dir,Nom,Ext;
  {
  cadena Respuesta, Aux;

  Aux = Fch_Nombre (Nom,Ext);
  Respuesta = Cad_Une (Dir,Aux,NIL);
  Cad_Destruye (Aux);

  return ( Respuesta );
  }

/*--------------------------------------------------------------------
 * FUNCION:  PideDirectorio()
 * OBJETIVO: Pedir al usuario un directorio
 * ENTRADAS: El directorio propuesto
 * SALIDAS:  El nuevo directorio
 * NOTA:     La cadena hay que destruirla cuando no sea necesaria
 *------------------------------------------------------------------*/
cadena PideDirectorio (Dir)
cadena Dir;
  {
  static contador Fil=13, Col=22, Ancho=36, Alto=6;

  Cdr_Caja (CDR_SIMPLE,Fil,Col,Fil+Alto,Col+Ancho,NEGRO,VERDE);

  Pan_Color (NEGRO,VERDE);
  Pan_PonTexto (Fil+2,Col+6,"Escribe el directorio:");
  return ( Usr_Texto (Dir,LONG_DIRECTORIO,Fil+4,Col+6,NEGRO,BLANCO) );
  }

/*--------------------------------------------------------------------
 * FUNCION:  PideExtension()
 * OBJETIVO: Pedir al usuario una extensin
 * ENTRADAS: La extensin propuesta
 * SALIDAS:  La nueva extensin
 * NOTA:     La cadena hay que destruirla cuando no sea necesaria
 *------------------------------------------------------------------*/
cadena PideExtension (Ext)
cadena Ext;
  {
  static contador Fil=13, Col=23, Ancho=34, Alto=6;

  Cdr_Caja (CDR_SIMPLE,Fil,Col,Fil+Alto,Col+Ancho,NEGRO,VERDE);

  Pan_Color (NEGRO,VERDE);
  Pan_PonTexto (Fil+2,Col+6,"Escribe la extensin:");
  return ( Usr_Texto (Ext,LONG_EXTENSION,Fil+4,Col+8,NEGRO,BLANCO) );
  }

/*--------------------------------------------------------------------
 * FUNCION:  PideMascara()
 * OBJETIVO: Pedir al usuario una mscara
 * ENTRADAS: La mscara propuesta
 * SALIDAS:  La nueva mscara
 * NOTA:     La cadena hay que destruirla cuando no sea necesaria
 *------------------------------------------------------------------*/
cadena PideMascara (Mas)
cadena Mas;
  {
  static contador Fil=13, Col=24, Ancho=32, Alto=6;

  Cdr_Caja (CDR_SIMPLE,Fil,Col,Fil+Alto,Col+Ancho,NEGRO,VERDE);

  Pan_Color (NEGRO,VERDE);
  Pan_PonTexto (Fil+2,Col+6,"Escribe la mscara:");
  return ( Usr_Texto (Mas,LONG_MASCARA,Fil+4,Col+8,NEGRO,BLANCO) );
  }

/*--------------------------------------------------------------------
 * Funciones de manejo de colores
 *------------------------------------------------------------------*/

/*--------------------------------------------------------------------
 * FUNCION:  AsignaPaletaDAC()
 * OBJETIVO: Cambiar el registro DAC asignado a un registro de paleta
 * ENTRADAS: El nmero de registro de paleta (de 0 a 15) y el del
 *           registro DAC (de 0 a 255)
 * SALIDAS:  Ninguna
 * NOTA:     No estoy seguro de la correccin de la documentacin
 *           de esta funcin
 *------------------------------------------------------------------*/
void AsignaPaletaDAC (Paleta, DAC)
octeto Paleta, DAC;
  {
  union REGS r;

  r.h.ah = 0x10;
  r.h.al = 0x0;
  r.h.bh = DAC;
  r.h.bl = Paleta;

  int86 (0x10,&r,&r);
  }

/*--------------------------------------------------------------------
 * FUNCION:  CambiaColorDAC()
 * OBJETIVO: Cambiar el color asociado a un registro DAC
 * ENTRADAS: El nmero de registro (de 0 a 255) y los valores RGB del
 *           color (de 0 a 63)
 * SALIDAS:  Ninguna
 *------------------------------------------------------------------*/
void CambiaColorDAC (Registro, Rojo, Verde, Azul)
octeto Registro, Rojo, Verde, Azul;
  {
  union REGS r;

  r.h.ah = 0x10;
  r.h.al = 0x10;
  r.x.bx = Registro;
  r.h.dh = Rojo;
  r.h.ch = Verde;
  r.h.cl = Azul;

  int86 (0x10,&r,&r);
  }

/*--------------------------------------------------------------------
 * FUNCION:  RestauraColor()
 * OBJETIVO: Redefinir los colores usados por Olimpo en modo grfico
 * ENTRADAS: Ninguna
 * SALIDAS:  Ninguna
 *------------------------------------------------------------------*/
void RestauraColor()
  {
  CambiaColorDAC (NEGRO,  0x00, 0x00, 0x00);
  CambiaColorDAC (ROJO,   0x2A, 0x00, 0x00);
  CambiaColorDAC (VERDE,  0x00, 0x2A, 0x00);
  CambiaColorDAC (BLANCO, 0x2A, 0x2A, 0x2A);
  }

/*--------------------------------------------------------------------
 * Funciones que extienden el objeto Tiff
 *
 *   Son necesarias para manejar imgenes de 16 colores
 *   Utilizan funciones internas del objeto Tiff
 *------------------------------------------------------------------*/

/*--------------------------------------------------------------------
 * FUNCION:  Tif_TablaColor()
 * OBJETIVO: Leer la tabla de 16 colores
 * ENTRADAS: Un tiff y un vector de 16 vectores de 3 valores RGB
 * SALIDAS:  Lgica, que indica que todo ha ido bien
 * EJEMPLO:  Tif_TablaColor ( Imagen, Tabla )
 *------------------------------------------------------------------*/
logico Tif_TablaColor (Tiff, Tabla)
tiff    Tiff;
contpos Tabla[16][3];
  {
  logico   Encontrado = NO, Respuesta = NO;
  contador Ancho, Alto, i, j, k=0;
  entero   Pos;
  contpos  Aux[48];

  for ( i=0 ; i<Tif_TotalTag(Tiff) && !Encontrado ; i++ )
    {
    if ( Tif_NumeroTag(Tiff,i) == 320 )
      {
      Pos = Tif_ValorTag(Tiff,i);
      Encontrado = SI;
      }
    }

  if ( Encontrado &&
       Fch_Coloca (Tif_Fichero(Tiff),Pos) &&
       Fch_LeeOcteto (Tif_Fichero(Tiff),(memoria)Aux,16*3*2) )
    {
    /* Creo que el orden de los datos depende del tag Photometric Interp
     * Esta rutina slo vale para TIFF formato Intel
     */
    for ( i=0 ; i<3  ; i++ )
      {
      for ( j=0 ; j<16 ; j++ )
        { Tabla[j][i] = Aux[k++]; }
      }
    Respuesta = SI;
    }

  return ( Respuesta );
  }

/*--------------------------------------------------------------------
 * FUNCION:  Tif_SituaEnDato()
 * OBJETIVO: Situar el puntero del fichero en la posicin en que comienzan
 *           los datos de la imagen
 * ENTRADAS: Un tiff
 * SALIDAS:  Lgica, que indica que todo ha ido bien
 * EJEMPLO:  Tif_SituaEnDato ( Imagen )
 * NOTA:     Esta funcin no es general, puesto que asume que la
 *           imagen slo tiene una banda
 *------------------------------------------------------------------*/
logico Tif_SituaEnDato (Tiff)
tiff    Tiff;
  {
  logico Respuesta = NO;
  entero Pos;

  Pos = Tif_Desplazamiento (Tiff, 0);
  if ( Pos && Fch_Coloca (Tif_Fichero(Tiff),Pos) )
    { Respuesta = SI; }

  return ( Respuesta );
  }

/*--------------------------------------------------------------------
 * FUNCION:  Tif_LeeLinea()
 * OBJETIVO: Leer una lnea de datos de la imagen
 * ENTRADAS: Un tiff y la direccin de memoria donde dejar los datos
 * SALIDAS:  Lgica, que indica que todo ha ido bien
 * EJEMPLO:  Tif_LeeLinea ( Imagen , Dato )
 * NOTAS:    El TIFF debe ser de 16 colores
 *           Se asume que el fichero est colocado en la posicin correcta
 *------------------------------------------------------------------*/
logico Tif_LeeLinea (Tiff, Dato)
tiff    Tiff;
memoria Dato;
  {
  logico   Respuesta = NO;
  memoria  Aux;
  contador Ancho, i, j=0;

  Ancho = (Tif_Ancho(Tiff) + 1) / 2;

  Aux = Mem_Crea (Ancho);

  if ( Fch_LeeOcteto (Tif_Fichero(Tiff), Aux, Ancho) )
    { Respuesta = SI; }

  if ( Respuesta )
    {
    for ( i=0 ; i<Ancho ; i++ )
      {
      Dato[j++] = Aux[i] >> 4;
      Dato[j++] = Aux[i] & 0x0F;
      }
    }

  Mem_Destruye (Aux);

  return ( Respuesta );
  }